use crate::store::nsobject::get_repo_root;
use std::path::PathBuf;
use crypto::digest::Digest;
use crypto::sha1::Sha1;
use std::ops::{Deref, DerefMut};
use crate::utils::path;

#[derive(Debug, PartialEq, Clone)]
pub struct ObjPath {
    path: PathBuf
}

impl Deref for ObjPath {
    type Target = PathBuf;
    fn deref(&self) -> &PathBuf { &self.path }
}

impl DerefMut for ObjPath {
    fn deref_mut(&mut self) -> &mut PathBuf { &mut self.path }
}

pub fn to_obj_path(path: &PathBuf) -> ObjPath {
    ObjPath { path: path.clone() }
}

impl Into<ObjPath> for &PathBuf {
    fn into(self) -> ObjPath {
        ObjPath { path: path::to_repo_relative(self, &get_repo_root())}
    }
}

impl Into<ObjPath> for PathBuf {
    fn into(self) -> ObjPath {
        ObjPath { path: path::to_repo_relative(&self, &get_repo_root())}
    }
}

#[derive(Debug, PartialEq)]
pub struct NsObjPath {
    path: PathBuf
}

impl Deref for NsObjPath {
    type Target = PathBuf;
    fn deref(&self) -> &PathBuf { &self.path }
}

impl DerefMut for NsObjPath {
    fn deref_mut(&mut self) -> &mut PathBuf { &mut self.path }
}

impl From<&str> for NsObjPath {
    fn from(hash: &str) -> Self {
        let (dir, res) = hash.split_at(2);

        let mut ns_obj_path = get_repo_root();
        ns_obj_path.push(dir);
        ns_obj_path.push(res);

        NsObjPath { path: ns_obj_path }
    }
}

impl From<&ObjPath> for NsObjPath {
    fn from(obj_path: &ObjPath) -> Self {
        let mut hasher = Sha1::new();
        hasher.input_str(
            obj_path
                .to_str()
                .expect("Cannot contains non UTF-8 char in path"),
        );
        NsObjPath::from(hasher.result_str().as_str())
    }
}
